/**
 * (c) 2014 Nucleus project. All rights reserved.
 * Released under GPL v2 license. Read LICENSE for more details.
 */

#pragma once

#include "nucleus/common.h"
#include "nucleus/cpu/translator.h"
#include "nucleus/cpu/ppu/ppu_thread.h"
#include "nucleus/cpu/ppu/ppu_fields.h"

class PPUInterpreter : public CellTranslator
{
    PPUThread& thread;

public:
    PPUInterpreter(PPUThread& thr);

    // Decode and execute one instruction
    void step();

    // Instruction callers
    static void callTable4  (PPUFields code, PPUThread& thread);
    static void callTable4_ (PPUFields code, PPUThread& thread);
    static void callTable19 (PPUFields code, PPUThread& thread);
    static void callTable30 (PPUFields code, PPUThread& thread);
    static void callTable31 (PPUFields code, PPUThread& thread);
    static void callTable58 (PPUFields code, PPUThread& thread);
    static void callTable59 (PPUFields code, PPUThread& thread);
    static void callTable62 (PPUFields code, PPUThread& thread);
    static void callTable63 (PPUFields code, PPUThread& thread);
    static void callTable63_(PPUFields code, PPUThread& thread);

    /**
     * Auxiliary functions
     */
    static inline bool isCarry(u64 a, u64 b);
    static inline bool isCarry(u64 a, u64 b, u64 c);
    static inline f32 CheckVSCR_NJ(PPUThread& thread, const f32 v);
    static inline bool CheckCondition(PPUThread& thread, u32 bo, u32 bi);
    static inline u64& GetRegBySPR(PPUThread& thread, u32 spr);

    static void add(PPUFields code, PPUThread& thread);
    static void addc(PPUFields code, PPUThread& thread);
    static void adde(PPUFields code, PPUThread& thread);
    static void addi(PPUFields code, PPUThread& thread);
    static void addic(PPUFields code, PPUThread& thread);
    static void addic_(PPUFields code, PPUThread& thread);
    static void addis(PPUFields code, PPUThread& thread);
    static void addme(PPUFields code, PPUThread& thread);
    static void addze(PPUFields code, PPUThread& thread);
    static void andx(PPUFields code, PPUThread& thread);
    static void andc(PPUFields code, PPUThread& thread);
    static void andi_(PPUFields code, PPUThread& thread);
    static void andis_(PPUFields code, PPUThread& thread);
    static void b(PPUFields code, PPUThread& thread);
    static void bc(PPUFields code, PPUThread& thread);
    static void bcctr(PPUFields code, PPUThread& thread);
    static void bclr(PPUFields code, PPUThread& thread);
    static void cmp(PPUFields code, PPUThread& thread);
    static void cmpi(PPUFields code, PPUThread& thread);
    static void cmpl(PPUFields code, PPUThread& thread);
    static void cmpli(PPUFields code, PPUThread& thread);
    static void cntlzd(PPUFields code, PPUThread& thread);
    static void cntlzw(PPUFields code, PPUThread& thread);
    static void crand(PPUFields code, PPUThread& thread);
    static void crandc(PPUFields code, PPUThread& thread);
    static void creqv(PPUFields code, PPUThread& thread);
    static void crnand(PPUFields code, PPUThread& thread);
    static void crnor(PPUFields code, PPUThread& thread);
    static void cror(PPUFields code, PPUThread& thread);
    static void crorc(PPUFields code, PPUThread& thread);
    static void crxor(PPUFields code, PPUThread& thread);
    static void dcbf(PPUFields code, PPUThread& thread);
    static void dcbst(PPUFields code, PPUThread& thread);
    static void dcbt(PPUFields code, PPUThread& thread);
    static void dcbtst(PPUFields code, PPUThread& thread);
    static void dcbz(PPUFields code, PPUThread& thread);
    static void divd(PPUFields code, PPUThread& thread);
    static void divdu(PPUFields code, PPUThread& thread);
    static void divw(PPUFields code, PPUThread& thread);
    static void divwu(PPUFields code, PPUThread& thread);
    static void dss(PPUFields code, PPUThread& thread);
    static void dst(PPUFields code, PPUThread& thread);
    static void dstst(PPUFields code, PPUThread& thread);
    static void eciwx(PPUFields code, PPUThread& thread);
    static void ecowx(PPUFields code, PPUThread& thread);
    static void eieio(PPUFields code, PPUThread& thread);
    static void eqv(PPUFields code, PPUThread& thread);
    static void extsb(PPUFields code, PPUThread& thread);
    static void extsh(PPUFields code, PPUThread& thread);
    static void extsw(PPUFields code, PPUThread& thread);
    static void icbi(PPUFields code, PPUThread& thread);
    static void isync(PPUFields code, PPUThread& thread);
    static void lbz(PPUFields code, PPUThread& thread);
    static void lbzu(PPUFields code, PPUThread& thread);
    static void lbzux(PPUFields code, PPUThread& thread);
    static void lbzx(PPUFields code, PPUThread& thread);
    static void ld(PPUFields code, PPUThread& thread);
    static void ldarx(PPUFields code, PPUThread& thread);
    static void ldbrx(PPUFields code, PPUThread& thread);
    static void ldu(PPUFields code, PPUThread& thread);
    static void ldux(PPUFields code, PPUThread& thread);
    static void ldx(PPUFields code, PPUThread& thread);
    static void lha(PPUFields code, PPUThread& thread);
    static void lhau(PPUFields code, PPUThread& thread);
    static void lhaux(PPUFields code, PPUThread& thread);
    static void lhax(PPUFields code, PPUThread& thread);
    static void lhbrx(PPUFields code, PPUThread& thread);
    static void lhz(PPUFields code, PPUThread& thread);
    static void lhzu(PPUFields code, PPUThread& thread);
    static void lhzux(PPUFields code, PPUThread& thread);
    static void lhzx(PPUFields code, PPUThread& thread);
    static void lmw(PPUFields code, PPUThread& thread);
    static void lswi(PPUFields code, PPUThread& thread);
    static void lswx(PPUFields code, PPUThread& thread);
    static void lwa(PPUFields code, PPUThread& thread);
    static void lwarx(PPUFields code, PPUThread& thread);
    static void lwaux(PPUFields code, PPUThread& thread);
    static void lwax(PPUFields code, PPUThread& thread);
    static void lwbrx(PPUFields code, PPUThread& thread);
    static void lwz(PPUFields code, PPUThread& thread);
    static void lwzu(PPUFields code, PPUThread& thread);
    static void lwzux(PPUFields code, PPUThread& thread);
    static void lwzx(PPUFields code, PPUThread& thread);
    static void mcrf(PPUFields code, PPUThread& thread);
    static void mcrfs(PPUFields code, PPUThread& thread);
    static void mfocrf(PPUFields code, PPUThread& thread);
    static void mfspr(PPUFields code, PPUThread& thread);
    static void mftb(PPUFields code, PPUThread& thread);
    static void mtfsb0(PPUFields code, PPUThread& thread);
    static void mtfsb1(PPUFields code, PPUThread& thread);
    static void mtfsfi(PPUFields code, PPUThread& thread);
    static void mtocrf(PPUFields code, PPUThread& thread);
    static void mtspr(PPUFields code, PPUThread& thread);
    static void mulhd(PPUFields code, PPUThread& thread);
    static void mulhdu(PPUFields code, PPUThread& thread);
    static void mulhw(PPUFields code, PPUThread& thread);
    static void mulhwu(PPUFields code, PPUThread& thread);
    static void mulld(PPUFields code, PPUThread& thread);
    static void mulli(PPUFields code, PPUThread& thread);
    static void mullw(PPUFields code, PPUThread& thread);
    static void nand(PPUFields code, PPUThread& thread);
    static void neg(PPUFields code, PPUThread& thread);
    static void nop(PPUFields code, PPUThread& thread);
    static void nor(PPUFields code, PPUThread& thread);
    static void orx(PPUFields code, PPUThread& thread);
    static void orc(PPUFields code, PPUThread& thread);
    static void ori(PPUFields code, PPUThread& thread);
    static void oris(PPUFields code, PPUThread& thread);
    static void rldc_lr(PPUFields code, PPUThread& thread);
    static void rldicx(PPUFields code, PPUThread& thread);
    static void rldiclx(PPUFields code, PPUThread& thread);
    static void rldicrx(PPUFields code, PPUThread& thread);
    static void rldimix(PPUFields code, PPUThread& thread);
    static void rlwimix(PPUFields code, PPUThread& thread);
    static void rlwinmx(PPUFields code, PPUThread& thread);
    static void rlwnmx(PPUFields code, PPUThread& thread);
    static void sc(PPUFields code, PPUThread& thread_code);
    static void sld(PPUFields code, PPUThread& thread);
    static void slw(PPUFields code, PPUThread& thread);
    static void srad(PPUFields code, PPUThread& thread);
    static void sradi(PPUFields code, PPUThread& thread);
    static void sraw(PPUFields code, PPUThread& thread);
    static void srawi(PPUFields code, PPUThread& thread);
    static void srd(PPUFields code, PPUThread& thread);
    static void srw(PPUFields code, PPUThread& thread);
    static void stb(PPUFields code, PPUThread& thread);
    static void stbu(PPUFields code, PPUThread& thread);
    static void stbux(PPUFields code, PPUThread& thread);
    static void stbx(PPUFields code, PPUThread& thread);
    static void std(PPUFields code, PPUThread& thread);
    static void stdcx_(PPUFields code, PPUThread& thread);
    static void stdu(PPUFields code, PPUThread& thread);
    static void stdux(PPUFields code, PPUThread& thread);
    static void stdx(PPUFields code, PPUThread& thread);
    static void sth(PPUFields code, PPUThread& thread);
    static void sthbrx(PPUFields code, PPUThread& thread);
    static void sthu(PPUFields code, PPUThread& thread);
    static void sthux(PPUFields code, PPUThread& thread);
    static void sthx(PPUFields code, PPUThread& thread);
    static void stmw(PPUFields code, PPUThread& thread);
    static void stswi(PPUFields code, PPUThread& thread);
    static void stswx(PPUFields code, PPUThread& thread);
    static void stvrxl(PPUFields code, PPUThread& thread);
    static void stw(PPUFields code, PPUThread& thread);
    static void stwbrx(PPUFields code, PPUThread& thread);
    static void stwcx_(PPUFields code, PPUThread& thread);
    static void stwu(PPUFields code, PPUThread& thread);
    static void stwux(PPUFields code, PPUThread& thread);
    static void stwx(PPUFields code, PPUThread& thread);
    static void subf(PPUFields code, PPUThread& thread);
    static void subfc(PPUFields code, PPUThread& thread);
    static void subfe(PPUFields code, PPUThread& thread);
    static void subfic(PPUFields code, PPUThread& thread);
    static void subfme(PPUFields code, PPUThread& thread);
    static void subfze(PPUFields code, PPUThread& thread);
    static void sync(PPUFields code, PPUThread& thread);
    static void td(PPUFields code, PPUThread& thread);
    static void tdi(PPUFields code, PPUThread& thread);
    static void tw(PPUFields code, PPUThread& thread);
    static void twi(PPUFields code, PPUThread& thread);
    static void xorx(PPUFields code, PPUThread& thread);
    static void xori(PPUFields code, PPUThread& thread);
    static void xoris(PPUFields code, PPUThread& thread);

    // Floating-Point Unit instructions
    static void lfd(PPUFields code, PPUThread& thread);
    static void lfdu(PPUFields code, PPUThread& thread);
    static void lfdux(PPUFields code, PPUThread& thread);
    static void lfdx(PPUFields code, PPUThread& thread);
    static void lfs(PPUFields code, PPUThread& thread);
    static void lfsu(PPUFields code, PPUThread& thread);
    static void lfsux(PPUFields code, PPUThread& thread);
    static void lfsx(PPUFields code, PPUThread& thread);
    static void mffs(PPUFields code, PPUThread& thread);
    static void mtfsf(PPUFields code, PPUThread& thread);
    static void fabs(PPUFields code, PPUThread& thread);
    static void fadd(PPUFields code, PPUThread& thread);
    static void fadds(PPUFields code, PPUThread& thread);
    static void fcfid(PPUFields code, PPUThread& thread);
    static void fcmpo(PPUFields code, PPUThread& thread);
    static void fcmpu(PPUFields code, PPUThread& thread);
    static void fctid(PPUFields code, PPUThread& thread);
    static void fctidz(PPUFields code, PPUThread& thread);
    static void fctiw(PPUFields code, PPUThread& thread);
    static void fctiwz(PPUFields code, PPUThread& thread);
    static void fdiv(PPUFields code, PPUThread& thread);
    static void fdivs(PPUFields code, PPUThread& thread);
    static void fmadd(PPUFields code, PPUThread& thread);
    static void fmadds(PPUFields code, PPUThread& thread);
    static void fmr(PPUFields code, PPUThread& thread);
    static void fmsub(PPUFields code, PPUThread& thread);
    static void fmsubs(PPUFields code, PPUThread& thread);
    static void fmul(PPUFields code, PPUThread& thread);
    static void fmuls(PPUFields code, PPUThread& thread);
    static void fnabs(PPUFields code, PPUThread& thread);
    static void fneg(PPUFields code, PPUThread& thread);
    static void fnmadd(PPUFields code, PPUThread& thread);
    static void fnmadds(PPUFields code, PPUThread& thread);
    static void fnmsub(PPUFields code, PPUThread& thread);
    static void fnmsubs(PPUFields code, PPUThread& thread);
    static void fres(PPUFields code, PPUThread& thread);
    static void frsp(PPUFields code, PPUThread& thread);
    static void frsqrte(PPUFields code, PPUThread& thread);
    static void fsel(PPUFields code, PPUThread& thread);
    static void fsqrt(PPUFields code, PPUThread& thread);
    static void fsqrts(PPUFields code, PPUThread& thread);
    static void fsub(PPUFields code, PPUThread& thread);
    static void fsubs(PPUFields code, PPUThread& thread);
    static void stfd(PPUFields code, PPUThread& thread);
    static void stfdu(PPUFields code, PPUThread& thread);
    static void stfdux(PPUFields code, PPUThread& thread);
    static void stfdx(PPUFields code, PPUThread& thread);
    static void stfiwx(PPUFields code, PPUThread& thread);
    static void stfs(PPUFields code, PPUThread& thread);
    static void stfsu(PPUFields code, PPUThread& thread);
    static void stfsux(PPUFields code, PPUThread& thread);
    static void stfsx(PPUFields code, PPUThread& thread);

    // Vector/SIMD instructions
    static void lvebx(PPUFields code, PPUThread& thread);
    static void lvehx(PPUFields code, PPUThread& thread);
    static void lvewx(PPUFields code, PPUThread& thread);
    static void lvlx(PPUFields code, PPUThread& thread);
    static void lvlxl(PPUFields code, PPUThread& thread);
    static void lvrx(PPUFields code, PPUThread& thread);
    static void lvrxl(PPUFields code, PPUThread& thread);
    static void lvsl(PPUFields code, PPUThread& thread);
    static void lvsr(PPUFields code, PPUThread& thread);
    static void lvx(PPUFields code, PPUThread& thread);
    static void lvxl(PPUFields code, PPUThread& thread);
    static void mfvscr(PPUFields code, PPUThread& thread);
    static void mtvscr(PPUFields code, PPUThread& thread);
    static void stvebx(PPUFields code, PPUThread& thread);
    static void stvehx(PPUFields code, PPUThread& thread);
    static void stvewx(PPUFields code, PPUThread& thread);
    static void stvlx(PPUFields code, PPUThread& thread);
    static void stvlxl(PPUFields code, PPUThread& thread);
    static void stvrx(PPUFields code, PPUThread& thread);
    static void stvx(PPUFields code, PPUThread& thread);
    static void stvxl(PPUFields code, PPUThread& thread);
    static void vaddcuw(PPUFields code, PPUThread& thread);
    static void vaddfp(PPUFields code, PPUThread& thread);
    static void vaddsbs(PPUFields code, PPUThread& thread);
    static void vaddshs(PPUFields code, PPUThread& thread);
    static void vaddsws(PPUFields code, PPUThread& thread);
    static void vaddubm(PPUFields code, PPUThread& thread);
    static void vaddubs(PPUFields code, PPUThread& thread);
    static void vadduhm(PPUFields code, PPUThread& thread);
    static void vadduhs(PPUFields code, PPUThread& thread);
    static void vadduwm(PPUFields code, PPUThread& thread);
    static void vadduws(PPUFields code, PPUThread& thread);
    static void vand(PPUFields code, PPUThread& thread);
    static void vandc(PPUFields code, PPUThread& thread);
    static void vavgsb(PPUFields code, PPUThread& thread);
    static void vavgsh(PPUFields code, PPUThread& thread);
    static void vavgsw(PPUFields code, PPUThread& thread);
    static void vavgub(PPUFields code, PPUThread& thread);
    static void vavguh(PPUFields code, PPUThread& thread);
    static void vavguw(PPUFields code, PPUThread& thread);
    static void vcfsx(PPUFields code, PPUThread& thread);
    static void vcfux(PPUFields code, PPUThread& thread);
    static void vcmpbfp(PPUFields code, PPUThread& thread);
    static void vcmpbfp_(PPUFields code, PPUThread& thread);
    static void vcmpeqfp(PPUFields code, PPUThread& thread);
    static void vcmpeqfp_(PPUFields code, PPUThread& thread);
    static void vcmpequb(PPUFields code, PPUThread& thread);
    static void vcmpequb_(PPUFields code, PPUThread& thread);
    static void vcmpequh(PPUFields code, PPUThread& thread);
    static void vcmpequh_(PPUFields code, PPUThread& thread);
    static void vcmpequw(PPUFields code, PPUThread& thread);
    static void vcmpequw_(PPUFields code, PPUThread& thread);
    static void vcmpgefp(PPUFields code, PPUThread& thread);
    static void vcmpgefp_(PPUFields code, PPUThread& thread);
    static void vcmpgtfp(PPUFields code, PPUThread& thread);
    static void vcmpgtfp_(PPUFields code, PPUThread& thread);
    static void vcmpgtsb(PPUFields code, PPUThread& thread);
    static void vcmpgtsb_(PPUFields code, PPUThread& thread);
    static void vcmpgtsh(PPUFields code, PPUThread& thread);
    static void vcmpgtsh_(PPUFields code, PPUThread& thread);
    static void vcmpgtsw(PPUFields code, PPUThread& thread);
    static void vcmpgtsw_(PPUFields code, PPUThread& thread);
    static void vcmpgtub(PPUFields code, PPUThread& thread);
    static void vcmpgtub_(PPUFields code, PPUThread& thread);
    static void vcmpgtuh(PPUFields code, PPUThread& thread);
    static void vcmpgtuh_(PPUFields code, PPUThread& thread);
    static void vcmpgtuw(PPUFields code, PPUThread& thread);
    static void vcmpgtuw_(PPUFields code, PPUThread& thread);
    static void vctsxs(PPUFields code, PPUThread& thread);
    static void vctuxs(PPUFields code, PPUThread& thread);
    static void vexptefp(PPUFields code, PPUThread& thread);
    static void vlogefp(PPUFields code, PPUThread& thread);
    static void vmaddfp(PPUFields code, PPUThread& thread);
    static void vmaxfp(PPUFields code, PPUThread& thread);
    static void vmaxsb(PPUFields code, PPUThread& thread);
    static void vmaxsh(PPUFields code, PPUThread& thread);
    static void vmaxsw(PPUFields code, PPUThread& thread);
    static void vmaxub(PPUFields code, PPUThread& thread);
    static void vmaxuh(PPUFields code, PPUThread& thread);
    static void vmaxuw(PPUFields code, PPUThread& thread);
    static void vmhaddshs(PPUFields code, PPUThread& thread);
    static void vmhraddshs(PPUFields code, PPUThread& thread);
    static void vminfp(PPUFields code, PPUThread& thread);
    static void vminsb(PPUFields code, PPUThread& thread);
    static void vminsh(PPUFields code, PPUThread& thread);
    static void vminsw(PPUFields code, PPUThread& thread);
    static void vminub(PPUFields code, PPUThread& thread);
    static void vminuh(PPUFields code, PPUThread& thread);
    static void vminuw(PPUFields code, PPUThread& thread);
    static void vmladduhm(PPUFields code, PPUThread& thread);
    static void vmrghb(PPUFields code, PPUThread& thread);
    static void vmrghh(PPUFields code, PPUThread& thread);
    static void vmrghw(PPUFields code, PPUThread& thread);
    static void vmrglb(PPUFields code, PPUThread& thread);
    static void vmrglh(PPUFields code, PPUThread& thread);
    static void vmrglw(PPUFields code, PPUThread& thread);
    static void vmsummbm(PPUFields code, PPUThread& thread);
    static void vmsumshm(PPUFields code, PPUThread& thread);
    static void vmsumshs(PPUFields code, PPUThread& thread);
    static void vmsumubm(PPUFields code, PPUThread& thread);
    static void vmsumuhm(PPUFields code, PPUThread& thread);
    static void vmsumuhs(PPUFields code, PPUThread& thread);
    static void vmulesb(PPUFields code, PPUThread& thread);
    static void vmulesh(PPUFields code, PPUThread& thread);
    static void vmuleub(PPUFields code, PPUThread& thread);
    static void vmuleuh(PPUFields code, PPUThread& thread);
    static void vmulosb(PPUFields code, PPUThread& thread);
    static void vmulosh(PPUFields code, PPUThread& thread);
    static void vmuloub(PPUFields code, PPUThread& thread);
    static void vmulouh(PPUFields code, PPUThread& thread);
    static void vnmsubfp(PPUFields code, PPUThread& thread);
    static void vnor(PPUFields code, PPUThread& thread);
    static void vor(PPUFields code, PPUThread& thread);
    static void vperm(PPUFields code, PPUThread& thread);
    static void vpkpx(PPUFields code, PPUThread& thread);
    static void vpkshss(PPUFields code, PPUThread& thread);
    static void vpkshus(PPUFields code, PPUThread& thread);
    static void vpkswss(PPUFields code, PPUThread& thread);
    static void vpkswus(PPUFields code, PPUThread& thread);
    static void vpkuhum(PPUFields code, PPUThread& thread);
    static void vpkuhus(PPUFields code, PPUThread& thread);
    static void vpkuwum(PPUFields code, PPUThread& thread);
    static void vpkuwus(PPUFields code, PPUThread& thread);
    static void vrefp(PPUFields code, PPUThread& thread);
    static void vrfim(PPUFields code, PPUThread& thread);
    static void vrfin(PPUFields code, PPUThread& thread);
    static void vrfip(PPUFields code, PPUThread& thread);
    static void vrfiz(PPUFields code, PPUThread& thread);
    static void vrlb(PPUFields code, PPUThread& thread);
    static void vrlh(PPUFields code, PPUThread& thread);
    static void vrlw(PPUFields code, PPUThread& thread);
    static void vrsqrtefp(PPUFields code, PPUThread& thread);
    static void vsel(PPUFields code, PPUThread& thread);
    static void vsl(PPUFields code, PPUThread& thread);
    static void vslb(PPUFields code, PPUThread& thread);
    static void vsldoi(PPUFields code, PPUThread& thread);
    static void vslh(PPUFields code, PPUThread& thread);
    static void vslo(PPUFields code, PPUThread& thread);
    static void vslw(PPUFields code, PPUThread& thread);
    static void vspltb(PPUFields code, PPUThread& thread);
    static void vsplth(PPUFields code, PPUThread& thread);
    static void vspltisb(PPUFields code, PPUThread& thread);
    static void vspltish(PPUFields code, PPUThread& thread);
    static void vspltisw(PPUFields code, PPUThread& thread);
    static void vspltw(PPUFields code, PPUThread& thread);
    static void vsr(PPUFields code, PPUThread& thread);
    static void vsrab(PPUFields code, PPUThread& thread);
    static void vsrah(PPUFields code, PPUThread& thread);
    static void vsraw(PPUFields code, PPUThread& thread);
    static void vsrb(PPUFields code, PPUThread& thread);
    static void vsrh(PPUFields code, PPUThread& thread);
    static void vsro(PPUFields code, PPUThread& thread);
    static void vsrw(PPUFields code, PPUThread& thread);
    static void vsubcuw(PPUFields code, PPUThread& thread);
    static void vsubfp(PPUFields code, PPUThread& thread);
    static void vsubsbs(PPUFields code, PPUThread& thread);
    static void vsubshs(PPUFields code, PPUThread& thread);
    static void vsubsws(PPUFields code, PPUThread& thread);
    static void vsububm(PPUFields code, PPUThread& thread);
    static void vsububs(PPUFields code, PPUThread& thread);
    static void vsubuhm(PPUFields code, PPUThread& thread);
    static void vsubuhs(PPUFields code, PPUThread& thread);
    static void vsubuwm(PPUFields code, PPUThread& thread);
    static void vsubuws(PPUFields code, PPUThread& thread);
    static void vsum2sws(PPUFields code, PPUThread& thread);
    static void vsum4sbs(PPUFields code, PPUThread& thread);
    static void vsum4shs(PPUFields code, PPUThread& thread);
    static void vsum4ubs(PPUFields code, PPUThread& thread);
    static void vsumsws(PPUFields code, PPUThread& thread);
    static void vupkhpx(PPUFields code, PPUThread& thread);
    static void vupkhsb(PPUFields code, PPUThread& thread);
    static void vupkhsh(PPUFields code, PPUThread& thread);
    static void vupklpx(PPUFields code, PPUThread& thread);
    static void vupklsb(PPUFields code, PPUThread& thread);
    static void vupklsh(PPUFields code, PPUThread& thread);
    static void vxor(PPUFields code, PPUThread& thread);

    // Unknown instruction
    static void unknown(PPUFields code, PPUThread& thread);
    static void unknown(const char* instruction);
};
